"""sportly.espn.hockey — NHL and college ice hockey via ESPN."""
from __future__ import annotations
from typing import Any
from sportly.client import SportlyClient
from sportly.espn._domains import ESPNDomain
from sportly.espn._leagues import LEAGUES as _ALL_LEAGUES
from sportly.espn.endpoints import advanced as _advanced_ep
from sportly.espn.endpoints import athletes as _athletes_ep
from sportly.espn.endpoints import news as _news_ep
from sportly.espn.endpoints import schedule as _schedule_ep
from sportly.espn.endpoints import standings as _standings_ep
from sportly.espn.endpoints import teams as _teams_ep
from sportly.models import Article, Athlete, Game, Standings, Team

SPORT = "hockey"
DEFAULT_LEAGUE = "nhl"
LEAGUES: dict[str, str] = _ALL_LEAGUES[SPORT]

def teams(league: str = DEFAULT_LEAGUE, *, limit: int = 100, client: SportlyClient | None = None) -> list[Team]:
    return _teams_ep.all(SPORT, league, limit=limit, client=client)

def team(team_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Team:
    return _teams_ep.one(SPORT, league, team_id, client=client)

def roster(team_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[Athlete]:
    return _teams_ep.roster(SPORT, league, team_id, client=client)

def scoreboard(league: str = DEFAULT_LEAGUE, *, date: str | None = None, limit: int = 100, client: SportlyClient | None = None) -> list[Game]:
    return _schedule_ep.scoreboard(SPORT, league, date=date, limit=limit, client=client)

def game(game_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Game:
    return _schedule_ep.game(SPORT, league, game_id, client=client)

def news(league: str = DEFAULT_LEAGUE, *, limit: int = 25, client: SportlyClient | None = None) -> list[Article]:
    return _news_ep.news(SPORT, league, limit=limit, client=client)

def standings(league: str = DEFAULT_LEAGUE, *, season: int | None = None, client: SportlyClient | None = None) -> Standings:
    return _standings_ep.standings(SPORT, league, season=season, client=client)

def injuries(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    return _advanced_ep.injuries(SPORT, league, client=client)

def transactions(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    return _advanced_ep.transactions(SPORT, league, client=client)

def athlete(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Athlete:
    return _athletes_ep.athlete(SPORT, league, athlete_id, client=client)

def play_by_play(event_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    return _advanced_ep.play_by_play(SPORT, league, event_id, client=client)



"""sportly.mlb.endpoints.schedule — Schedule endpoints."""
from __future__ import annotations
from typing import Any
from sportly.mlb._client import MLBClient, SPORT_ID, get_client


def today(
    *,
    hydrate: str | None = None,
    sport_id: int = SPORT_ID,
    client: MLBClient | None = None,
) -> list[dict[str, Any]]:
    """Return all games scheduled for today."""
    return games(sport_id=sport_id, hydrate=hydrate, client=client)


def games(
    *,
    date: str | None = None,
    start_date: str | None = None,
    end_date: str | None = None,
    team_id: int | None = None,
    sport_id: int = SPORT_ID,
    season: int | None = None,
    game_pk: int | None = None,
    hydrate: str | None = None,
    client: MLBClient | None = None,
) -> list[dict[str, Any]]:
    """Return games for a date, date range, or team.

    Parameters
    ----------
    date:       Single date ``YYYY-MM-DD``.
    start_date: Range start ``YYYY-MM-DD``.
    end_date:   Range end ``YYYY-MM-DD``.
    team_id:    Filter to one team.
    hydrate:    Embed sub-resources, e.g. ``"team,linescore,probablePitcher"``.

    Examples
    --------
    ::

        from sportly.mlb.endpoints import schedule

        # Today
        schedule.today()

        # April 1, 2025
        schedule.games(date="2025-04-01")

        # Dodgers full April
        schedule.games(team_id=119, start_date="2025-04-01", end_date="2025-04-30")

        # With probable pitchers
        schedule.games(date="2025-04-01", hydrate="team,linescore,probablePitcher")
    """
    http = client or get_client()
    data = http.get(
        "schedule",
        sportId=sport_id,
        date=date,
        startDate=start_date,
        endDate=end_date,
        teamId=team_id,
        season=season,
        gamePk=game_pk,
        hydrate=hydrate,
    )
    result: list[dict] = []
    for day in data.get("dates", []):
        result.extend(day.get("games", []))
    return result  # type: ignore[return-value]



"""nhl.endpoints.games — boxscore, play-by-play, and shifts."""
from __future__ import annotations
from typing import Any
from sportly.nhl._client import NHLClient


def boxscore(game_id: str, *, client: NHLClient | None = None) -> dict[str, Any]:
    """Full boxscore for a game.

    Parameters
    ----------
    game_id: NHL game ID, e.g. ``"2024020001"``.

    Example::

        from sportly.nhl.endpoints import games
        box = games.boxscore("2024020001")
    """
    return (client or NHLClient())._get(f"/gamecenter/{game_id}/boxscore")


def play_by_play(game_id: str, *, client: NHLClient | None = None) -> dict[str, Any]:
    """All plays with x/y coordinates, shot types, and player references.

    Example::

        pbp = games.play_by_play("2024020001")
        for play in pbp["plays"]:
            print(play["typeDescKey"], play["timeInPeriod"])
    """
    return (client or NHLClient())._get(f"/gamecenter/{game_id}/play-by-play")


def landing(game_id: str, *, client: NHLClient | None = None) -> dict[str, Any]:
    """Game landing page data (used by NHL.com — includes live score, period summary)."""
    return (client or NHLClient())._get(f"/gamecenter/{game_id}/landing")



"""sportly.nhl — Native NHL Web API client.

Base URL: https://api-web.nhle.com/v1
No auth required.

Features not available through ESPN: play-by-play with coordinates,
shift charts, game landing pages, roster history, standings by date.

Usage::

    from sportly import nhl

    # Franchises
    all_teams = nhl.teams()

    # Schedule
    today    = nhl.schedule()
    specific = nhl.schedule("2024-04-15")

    # Games
    box = nhl.game("2024020001")
    pbp = nhl.play_by_play("2024020001")

    # Teams
    roster   = nhl.roster("TOR")
    standings = nhl.standings()

    # Players
    mcdavid = nhl.player(8481528)

Endpoint subpackage::

    from sportly.nhl.endpoints import games, schedule, teams
"""
from __future__ import annotations
from typing import Any

from sportly.nhl._client import NHLClient
from sportly.nhl.endpoints import games as _games_ep
from sportly.nhl.endpoints import schedule as _sched_ep
from sportly.nhl.endpoints import teams as _teams_ep

__all__ = [
    "NHLClient",
    "teams", "schedule", "game", "play_by_play", "landing",
    "roster", "standings", "player",
]

_inst = NHLClient()

# ── Teams / Franchises ────────────────────────────────────────────────────────

def teams() -> list[dict[str, Any]]:
    """All NHL franchises."""
    return _teams_ep.franchises(client=_inst)


def roster(team_abbrev: str, season: str | None = None) -> dict[str, Any]:
    """Current (or historical) roster for a team (e.g. ``"TOR"``)."""
    return _teams_ep.roster(team_abbrev, season, client=_inst)


def standings(*, date: str | None = None) -> dict[str, Any]:
    """League standings — current or as of a specific ``YYYY-MM-DD`` date."""
    return _teams_ep.standings(date=date, client=_inst)


def player(player_id: int | str) -> dict[str, Any]:
    """Player profile and career stats by numeric NHL player ID."""
    return _teams_ep.player(player_id, client=_inst)


# ── Schedule ──────────────────────────────────────────────────────────────────

def schedule(date: str | None = None) -> dict[str, Any]:
    """NHL schedule for a date (``YYYY-MM-DD``) or today."""
    return _sched_ep.schedule(date, client=_inst)


def weekly(date: str | None = None) -> dict[str, Any]:
    """Week-long schedule centred around a date."""
    return _sched_ep.weekly(date, client=_inst)


# ── Games ─────────────────────────────────────────────────────────────────────

def game(game_id: str) -> dict[str, Any]:
    """Full boxscore for a game."""
    return _games_ep.boxscore(game_id, client=_inst)


def play_by_play(game_id: str) -> dict[str, Any]:
    """All plays with x/y coordinates, shot types, and player references."""
    return _games_ep.play_by_play(game_id, client=_inst)


def landing(game_id: str) -> dict[str, Any]:
    """Game landing page (live score, period summary, used by NHL.com)."""
    return _games_ep.landing(game_id, client=_inst)



"""Tests for SportlyClient error handling."""

from __future__ import annotations

import pytest
import httpx

from sportly.client import SportlyClient
from sportly.exceptions import NotFoundError, RateLimitError, UpstreamError, SportlyError


class TestClientErrorMapping:
    """Verify HTTP status codes map to the correct exception types."""

    def _client_with_response(self, status_code: int, body: str = "{}") -> SportlyClient:
        """Return a SportlyClient that always responds with the given status."""
        transport = httpx.MockTransport(
            lambda request: httpx.Response(status_code, content=body.encode())
        )
        client = SportlyClient()
        client._http = httpx.Client(transport=transport)
        return client

    def test_404_raises_not_found(self):
        c = self._client_with_response(404)
        with pytest.raises(NotFoundError):
            c.get("https://example.com/api")

    def test_429_raises_rate_limit(self):
        c = self._client_with_response(429)
        with pytest.raises(RateLimitError):
            c.get("https://example.com/api")

    def test_500_raises_upstream_error(self):
        c = self._client_with_response(500)
        with pytest.raises(UpstreamError):
            c.get("https://example.com/api")

    def test_400_raises_sportly_error(self):
        c = self._client_with_response(400)
        with pytest.raises(SportlyError):
            c.get("https://example.com/api")

    def test_200_returns_dict(self):
        c = self._client_with_response(200, '{"hello": "world"}')
        result = c.get("https://example.com/api")
        assert result == {"hello": "world"}



"""sportly.espn.basketball — NBA, WNBA, college, and international basketball.

Usage::

    from sportly.espn import basketball

    # All NBA teams
    teams = basketball.teams()                        # default league: nba

    # WNBA scoreboard
    games = basketball.scoreboard("wnba")

    # Specific date
    games = basketball.scoreboard("nba", date="20241225")

    # v3 enriched scoreboard
    from sportly.espn._domains import ESPNDomain
    games = basketball.scoreboard("nba", domain=ESPNDomain.SITE_V3)

    # Lakers roster
    roster = basketball.roster("13")                  # team_id 13 = Lakers

    # LeBron stats
    overview = basketball.athlete_overview("1966")    # id 1966 = LeBron

    # NCAA Bracketology
    bracket = basketball.bracketology()

    # NBA BPI power index
    bpi = basketball.power_index("nba")
"""

from __future__ import annotations

from typing import Any

from sportly.client import SportlyClient, get_client
from sportly.espn._domains import ESPNDomain, build_url
from sportly.espn._leagues import LEAGUES as _ALL_LEAGUES
from sportly.espn.endpoints import athletes as _athletes_ep
from sportly.espn.endpoints import advanced as _advanced_ep
from sportly.espn.endpoints import news as _news_ep
from sportly.espn.endpoints import schedule as _schedule_ep
from sportly.espn.endpoints import standings as _standings_ep
from sportly.espn.endpoints import teams as _teams_ep
from sportly.models import Article, Athlete, Game, Standings, Team

SPORT = "basketball"
DEFAULT_LEAGUE = "nba"

#: All known ESPN basketball leagues. Keys are valid league slugs.
LEAGUES: dict[str, str] = _ALL_LEAGUES[SPORT]


# ── Common methods ─────────────────────────────────────────────────────────────

def teams(league: str = DEFAULT_LEAGUE, *, limit: int = 100, client: SportlyClient | None = None) -> list[Team]:
    """Return all teams. Defaults to NBA."""
    return _teams_ep.all(SPORT, league, limit=limit, client=client)


def team(team_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Team:
    """Return a single team by ESPN team ID (e.g. ``"13"`` = Lakers)."""
    return _teams_ep.one(SPORT, league, team_id, client=client)


def roster(team_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[Athlete]:
    """Return the roster for a team."""
    return _teams_ep.roster(SPORT, league, team_id, client=client)


def scoreboard(
    league: str = DEFAULT_LEAGUE,
    *,
    date: str | None = None,
    limit: int = 100,
    domain: ESPNDomain = ESPNDomain.SITE,
    client: SportlyClient | None = None,
) -> list[Game]:
    """Return games. Pass ``domain=ESPNDomain.SITE_V3`` for enriched data."""
    return _schedule_ep.scoreboard(SPORT, league, date=date, limit=limit, domain=domain, client=client)


def game(game_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Game:
    """Return full game summary."""
    return _schedule_ep.game(SPORT, league, game_id, client=client)


def news(league: str = DEFAULT_LEAGUE, *, limit: int = 25, client: SportlyClient | None = None) -> list[Article]:
    """Return latest news articles."""
    return _news_ep.news(SPORT, league, limit=limit, client=client)


def now_news(*, league: str | None = None, limit: int = 25, client: SportlyClient | None = None) -> list[Article]:
    """Real-time headlines from now.core.api.espn.com."""
    return _news_ep.now_news(SPORT, league=league, limit=limit, client=client)


def standings(league: str = DEFAULT_LEAGUE, *, season: int | None = None, client: SportlyClient | None = None) -> Standings:
    """Return league standings (uses /apis/v2/ — site/v2 returns a stub)."""
    return _standings_ep.standings(SPORT, league, season=season, client=client)


def rankings(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    """Return poll rankings (NCAA leagues only)."""
    return _standings_ep.rankings(SPORT, league, client=client)


def injuries(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    """Return league-wide injury report."""
    return _advanced_ep.injuries(SPORT, league, client=client)


def transactions(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    """Return recent signings, trades, and waivers."""
    return _advanced_ep.transactions(SPORT, league, client=client)


def athlete(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Athlete:
    """Return athlete profile from the core API."""
    return _athletes_ep.athlete(SPORT, league, athlete_id, client=client)


def athlete_overview(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    """Stats snapshot + next game + news (web/v3)."""
    return _athletes_ep.overview(SPORT, league, athlete_id, client=client)


def athlete_stats(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    """Season statistics (web/v3)."""
    return _athletes_ep.stats(SPORT, league, athlete_id, client=client)


def athlete_gamelog(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    """Game-by-game log (web/v3)."""
    return _athletes_ep.gamelog(SPORT, league, athlete_id, client=client)


def athlete_splits(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    """Home/away/opponent splits (web/v3)."""
    return _athletes_ep.splits(SPORT, league, athlete_id, client=client)


def leaders(league: str = DEFAULT_LEAGUE, *, season: int | None = None, client: SportlyClient | None = None) -> dict[str, Any]:
    """Statistical leaders (core API)."""
    return _athletes_ep.leaders(SPORT, league, season=season, client=client)


def odds(event_id: str, competition_id: str | None = None, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    """Betting odds for a game."""
    return _advanced_ep.odds(SPORT, league, event_id, competition_id, client=client)


def play_by_play(event_id: str, competition_id: str | None = None, league: str = DEFAULT_LEAGUE, *, limit: int = 300, client: SportlyClient | None = None) -> dict[str, Any]:
    """Play-by-play for a game."""
    return _advanced_ep.play_by_play(SPORT, league, event_id, competition_id, limit=limit, client=client)


def cdn_game(game_id: str, *, client: SportlyClient | None = None) -> dict[str, Any]:
    """Full CDN game package (drives, plays, odds, win probability)."""
    return _advanced_ep.cdn_game(SPORT, game_id, client=client)


# ── Basketball-specific ───────────────────────────────────────────────────────

def bracketology(tournament_id: str = "22", *, year: int | None = None, client: SportlyClient | None = None) -> dict[str, Any]:
    """NCAA Tournament bracket projections.

    Parameters
    ----------
    tournament_id: ``"22"`` = NCAA Men's, ``"23"`` = NCAA Women's.
    year: Tournament year. Omit for current.

    Example
    -------
    ::

        bracket = basketball.bracketology()              # Men's current
        womens  = basketball.bracketology("23")          # Women's
    """
    http = client or get_client()
    path = f"tournament/{tournament_id}/seasons/{year}/bracketology" if year else f"tournament/{tournament_id}/bracketology"
    url = build_url(ESPNDomain.CORE, path)
    return http.get(url)  # type: ignore[return-value]


def power_index(league: str = DEFAULT_LEAGUE, *, season: int | None = None, client: SportlyClient | None = None) -> dict[str, Any]:
    """ESPN BPI (Basketball Power Index) ratings.

    Example
    -------
    ::

        bpi = basketball.power_index("mens-college-basketball")
    """
    http = client or get_client()
    base = f"sports/{SPORT}/leagues/{league}"
    path = f"{base}/seasons/{season}/powerindex" if season else f"{base}/powerindex"
    url = build_url(ESPNDomain.CORE, path)
    return http.get(url)  # type: ignore[return-value]



"""sportly.mlb.endpoints — functional MLB API endpoint modules."""
from sportly.mlb.endpoints import games, players, schedule, stats, teams  # noqa: F401

__all__ = ["teams", "schedule", "players", "games", "stats"]



"""ESPN athlete endpoint functions."""

from __future__ import annotations

from typing import Any

from sportly.client import SportlyClient, get_client
from sportly.espn._domains import ESPNDomain, build_url
from sportly.espn._parse import parse_athlete
from sportly.models import Athlete


def athlete(
    sport: str,
    league: str,
    athlete_id: str | int,
    *,
    domain: ESPNDomain = ESPNDomain.CORE,
    client: SportlyClient | None = None,
) -> Athlete:
    """Return a single athlete profile.

    Parameters
    ----------
    domain: ``ESPNDomain.CORE`` (v2, default) or ``ESPNDomain.CORE_V3`` (enriched).
    """
    http = client or get_client()
    url = build_url(domain, f"sports/{sport}/leagues/{league}/athletes/{athlete_id}")
    return parse_athlete(http.get(url))


def overview(
    sport: str,
    league: str,
    athlete_id: str | int,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Stats snapshot, next game, and news for an athlete (web/v3).

    .. note:: Soccer returns minimal data from this endpoint.
    """
    http = client or get_client()
    url = build_url(ESPNDomain.WEB_V3, f"sports/{sport}/{league}/athletes/{athlete_id}/overview")
    return http.get(url)  # type: ignore[return-value]


def stats(
    sport: str,
    league: str,
    athlete_id: str | int,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Season statistics for an athlete (web/v3).

    Works for NFL, NBA, NHL, MLB. Returns 404 for Soccer.
    """
    http = client or get_client()
    url = build_url(ESPNDomain.WEB_V3, f"sports/{sport}/{league}/athletes/{athlete_id}/stats")
    return http.get(url)  # type: ignore[return-value]


def gamelog(
    sport: str,
    league: str,
    athlete_id: str | int,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Game-by-game log for an athlete (web/v3)."""
    http = client or get_client()
    url = build_url(ESPNDomain.WEB_V3, f"sports/{sport}/{league}/athletes/{athlete_id}/gamelog")
    return http.get(url)  # type: ignore[return-value]


def splits(
    sport: str,
    league: str,
    athlete_id: str | int,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Home / away / opponent splits for an athlete (web/v3)."""
    http = client or get_client()
    url = build_url(ESPNDomain.WEB_V3, f"sports/{sport}/{league}/athletes/{athlete_id}/splits")
    return http.get(url)  # type: ignore[return-value]


def leaders(
    sport: str,
    league: str,
    *,
    season: int | None = None,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Statistical leaders from the core API."""
    http = client or get_client()
    base = f"sports/{sport}/leagues/{league}"
    path = f"{base}/seasons/{season}/leaders" if season else f"{base}/leaders"
    return http.get(build_url(ESPNDomain.CORE, path))  # type: ignore[return-value]


def stats_leaderboard(
    sport: str,
    league: str,
    *,
    category: str | None = None,
    limit: int = 50,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Athletes ranked by stat category (web/v3 byathlete endpoint)."""
    http = client or get_client()
    url = build_url(ESPNDomain.WEB_V3, f"sports/{sport}/{league}/statistics/byathlete")
    params: dict[str, Any] = {"limit": limit}
    if category:
        params["category"] = category
    return http.get(url, params=params)  # type: ignore[return-value]



"""fantasy.endpoints.league — league-level data (teams, rosters, standings, draft)."""
from __future__ import annotations
from typing import Any
from sportly.fantasy.endpoints._client import FantasyClient, get_client


def fetch(
    game_code: str,
    *,
    league_id: int,
    season: int,
    views: list[str] | None = None,
    scoring_period_id: int | None = None,
    matchup_period_id: int | None = None,
    cookies: dict[str, str] | None = None,
) -> dict[str, Any]:
    """Raw league fetch with arbitrary view(s).

    Parameters
    ----------
    game_code:         ``"ffl"``, ``"fba"``, ``"flb"``, ``"fhl"``
    league_id:         ESPN Fantasy league ID
    season:            Season year
    views:             One or more view names (see :data:`sportly.fantasy.VIEWS`)
    scoring_period_id: Filter by week / day
    cookies:           ``{"espn_s2": "...", "SWID": "{...}"}`` for private leagues
    """
    http = get_client(cookies)
    path = f"{game_code}/seasons/{season}/segments/0/leagues/{league_id}"
    params: dict[str, Any] = {}
    if views:
        params["view"] = views
    if scoring_period_id is not None:
        params["scoringPeriodId"] = scoring_period_id
    if matchup_period_id is not None:
        params["matchupPeriodId"] = matchup_period_id
    return http.get(path, **params)  # type: ignore[return-value]


def teams(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """All fantasy teams — names, owners, records."""
    return fetch(game_code, league_id=league_id, season=season, views=["mTeam"], cookies=cookies).get("teams", [])  # type: ignore[return-value]


def roster(game_code: str, *, league_id: int, season: int, scoring_period_id: int | None = None, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """All team rosters with player entries and lineup slot assignments."""
    return fetch(game_code, league_id=league_id, season=season, views=["mRoster"], scoring_period_id=scoring_period_id, cookies=cookies).get("teams", [])  # type: ignore[return-value]


def standings(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """Teams with W/L/PF/PA standings."""
    return fetch(game_code, league_id=league_id, season=season, views=["mTeam", "mStandings"], cookies=cookies).get("teams", [])  # type: ignore[return-value]


def draft(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> dict[str, Any]:
    """Full draft history — picks, player IDs, round order."""
    return fetch(game_code, league_id=league_id, season=season, views=["mDraftDetail"], cookies=cookies).get("draftDetail", {})  # type: ignore[return-value]


def live_scoring(game_code: str, *, league_id: int, season: int, scoring_period_id: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """Live scoring for an active scoring period."""
    return fetch(game_code, league_id=league_id, season=season, views=["mBoxscore", "mLiveScoring", "mScoreboard"], scoring_period_id=scoring_period_id, cookies=cookies).get("schedule", [])  # type: ignore[return-value]


def transactions(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """All transactions: waiver claims, trades, FA adds/drops."""
    return fetch(game_code, league_id=league_id, season=season, views=["mTransactions2"], cookies=cookies).get("transactions", [])  # type: ignore[return-value]



"""sportly.nhl.endpoints — NHL Web API endpoint submodules.

Submodules
----------
- ``games``    — ``boxscore()``, ``play_by_play()``, ``landing()``
- ``schedule`` — ``schedule()``, ``weekly()``
- ``teams``    — ``franchises()``, ``roster()``, ``standings()``, ``player()``
"""
from sportly.nhl.endpoints import (  # noqa: F401
    games,
    schedule,
    teams,
)



"""Tests for the sportly CLI entry point."""

from __future__ import annotations
import io
import contextlib
import argparse

import pytest

from sportly.cli import _build_parser


class TestCLIParserExisting:
    def test_info_subcommand(self):
        args = _build_parser().parse_args(["info"])
        assert args.command == "info"

    def test_espn_basketball_teams(self):
        args = _build_parser().parse_args(["espn", "basketball", "teams", "--league", "nba"])
        assert args.sport == "basketball"
        assert args.method == "teams"
        assert args.league == "nba"

    def test_espn_football_scoreboard(self):
        args = _build_parser().parse_args(["espn", "football", "scoreboard"])
        assert args.sport == "football"
        assert args.method == "scoreboard"

    def test_nhl_teams(self):
        args = _build_parser().parse_args(["nhl", "teams"])
        assert args.nhl_cmd == "teams"

    def test_nhl_schedule_with_date(self):
        args = _build_parser().parse_args(["nhl", "schedule", "--date", "2024-04-15"])
        assert args.date == "2024-04-15"

    def test_espn_game_with_id(self):
        args = _build_parser().parse_args(["espn", "basketball", "game", "--id", "401671803", "--league", "nba"])
        assert args.id == "401671803"


class TestCLIParserMLB:
    def test_schedule_no_date(self):
        args = _build_parser().parse_args(["mlb", "schedule"])
        assert args.mlb_cmd == "schedule"
        assert getattr(args, "date", None) is None

    def test_schedule_with_date(self):
        args = _build_parser().parse_args(["mlb", "schedule", "--date", "2025-04-01"])
        assert args.date == "2025-04-01"

    def test_teams(self):
        args = _build_parser().parse_args(["mlb", "teams"])
        assert args.mlb_cmd == "teams"

    def test_player_id(self):
        args = _build_parser().parse_args(["mlb", "player", "660271"])
        assert args.id == "660271"

    def test_leaders(self):
        args = _build_parser().parse_args(["mlb", "leaders", "homeRuns", "--season", "2025"])
        assert args.category == "homeRuns"
        assert args.season == 2025

    def test_standings(self):
        args = _build_parser().parse_args(["mlb", "standings"])
        assert args.mlb_cmd == "standings"

    def test_boxscore(self):
        args = _build_parser().parse_args(["mlb", "boxscore", "745444"])
        assert args.mlb_cmd == "boxscore"


class TestCLIParserNBA:
    def test_scoreboard(self):
        args = _build_parser().parse_args(["nba", "scoreboard", "2025-03-26"])
        assert args.nba_cmd == "scoreboard"
        assert args.date == "2025-03-26"

    def test_teams(self):
        assert _build_parser().parse_args(["nba", "teams"]).nba_cmd == "teams"

    def test_leaders(self):
        args = _build_parser().parse_args(["nba", "leaders", "PTS", "--season", "2024-25"])
        assert args.stat == "PTS"
        assert args.season == "2024-25"

    def test_career(self):
        args = _build_parser().parse_args(["nba", "career", "2544"])
        assert args.nba_cmd == "career"
        assert args.id == "2544"

    def test_standings(self):
        args = _build_parser().parse_args(["nba", "standings", "--season", "2024-25"])
        assert args.nba_cmd == "standings"

    def test_shot_chart(self):
        args = _build_parser().parse_args(["nba", "shot-chart", "201939", "--season", "2024-25"])
        assert args.nba_cmd == "shot-chart"


class TestCLIParserNFL:
    def test_scoreboard_week(self):
        args = _build_parser().parse_args(["nfl", "scoreboard", "--week", "1", "--season", "2024"])
        assert args.nfl_cmd == "scoreboard"
        assert args.week == 1
        assert args.season == 2024

    def test_teams(self):
        assert _build_parser().parse_args(["nfl", "teams"]).nfl_cmd == "teams"

    def test_injuries(self):
        assert _build_parser().parse_args(["nfl", "injuries"]).nfl_cmd == "injuries"

    def test_qbr(self):
        assert _build_parser().parse_args(["nfl", "qbr"]).nfl_cmd == "qbr"

    def test_depth_chart(self):
        args = _build_parser().parse_args(["nfl", "depth-chart", "6"])
        assert args.nfl_cmd == "depth-chart"
        assert args.id == "6"

    def test_news(self):
        assert _build_parser().parse_args(["nfl", "news"]).nfl_cmd == "news"


class TestCLIParserFotMob:
    def test_matches(self):
        args = _build_parser().parse_args(["fotmob", "matches", "20260326"])
        assert args.fm_cmd == "matches"
        assert args.date == "20260326"

    def test_league(self):
        args = _build_parser().parse_args(["fotmob", "league", "47"])
        assert args.id == 47

    def test_team(self):
        args = _build_parser().parse_args(["fotmob", "team", "8456"])
        assert args.id == 8456

    def test_search(self):
        args = _build_parser().parse_args(["fotmob", "search", "haaland"])
        assert args.term == "haaland"

    def test_leagues(self):
        assert _build_parser().parse_args(["fotmob", "leagues"]).fm_cmd == "leagues"


class TestCLIParserFantasy:
    def test_teams(self):
        args = _build_parser().parse_args(
            ["fantasy", "teams", "ffl", "--league-id", "336358", "--season", "2025"]
        )
        assert args.fs_cmd == "teams"
        assert args.game == "ffl"
        assert args.league_id == 336358
        assert args.season == 2025

    def test_draft(self):
        args = _build_parser().parse_args(
            ["fantasy", "draft", "fba", "--league-id", "999", "--season", "2025"]
        )
        assert args.fs_cmd == "draft"
        assert args.game == "fba"

    def test_meta(self):
        args = _build_parser().parse_args(["fantasy", "meta", "ffl"])
        assert args.fs_cmd == "meta"
        assert args.game == "ffl"

    def test_league_with_views(self):
        args = _build_parser().parse_args(
            ["fantasy", "league", "ffl", "--league-id", "123", "--season", "2025",
             "--views", "mTeam,mRoster"]
        )
        assert args.views == "mTeam,mRoster"

    def test_private_cookies(self):
        args = _build_parser().parse_args(
            ["fantasy", "teams", "ffl", "--league-id", "123", "--season", "2025",
             "--espn-s2", "AEB123", "--swid", "{ABC}"]
        )
        assert args.espn_s2 == "AEB123"
        assert args.swid == "{ABC}"


class TestCLIParserSofascore:
    def test_matches(self):
        args = _build_parser().parse_args(["sofascore", "matches", "football", "2026-03-26"])
        assert args.ss_cmd == "matches"
        assert args.sport == "football"
        assert args.date == "2026-03-26"

    def test_match(self):
        args = _build_parser().parse_args(["sofascore", "match", "11352523"])
        assert args.ss_cmd == "match"
        assert args.id == 11352523

    def test_player(self):
        args = _build_parser().parse_args(["sofascore", "player", "814123"])
        assert args.id == 814123

    def test_squad(self):
        args = _build_parser().parse_args(["sofascore", "squad", "4705"])
        assert args.ss_cmd == "squad"

    def test_team(self):
        args = _build_parser().parse_args(["sofascore", "team", "4705"])
        assert args.ss_cmd == "team"


# ── Handler smoke tests ───────────────────────────────────────────────────────

class TestCLIHandlers:
    def test_info_prints_version(self, capsys):
        from sportly.cli import _cmd_info
        _cmd_info(argparse.Namespace())
        out = capsys.readouterr().out
        assert "sportly" in out
        assert "1.1.0" in out
        assert "fotmob" in out.lower()

    def test_mlb_teams_handler(self, monkeypatch):
        import sportly.mlb as mlb
        import sportly.cli as cli
        monkeypatch.setattr(mlb, "teams", lambda: [{"id": 119, "name": "Dodgers"}])
        args = _build_parser().parse_args(["mlb", "teams"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_mlb(args)
        assert "Dodgers" in out.getvalue()

    def test_mlb_standings_handler(self, monkeypatch):
        import sportly.mlb as mlb
        import sportly.cli as cli
        monkeypatch.setattr(mlb, "standings", lambda season=None: {"records": []})
        args = _build_parser().parse_args(["mlb", "standings"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_mlb(args)
        assert "records" in out.getvalue()

    def test_nfl_teams_handler(self, monkeypatch):
        import sportly.nfl as nfl
        import sportly.cli as cli
        monkeypatch.setattr(nfl, "teams", lambda limit=32: [{"id": 12, "name": "Chiefs"}])
        args = _build_parser().parse_args(["nfl", "teams"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_nfl(args)
        assert "Chiefs" in out.getvalue()

    def test_nfl_injuries_handler(self, monkeypatch):
        import sportly.nfl as nfl
        import sportly.cli as cli
        monkeypatch.setattr(nfl, "injuries", lambda: [{"team": "DAL"}])
        args = _build_parser().parse_args(["nfl", "injuries"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_nfl(args)
        assert "DAL" in out.getvalue()

    def test_fotmob_leagues_handler(self, capsys):
        from sportly.cli import _cmd_fotmob
        args = _build_parser().parse_args(["fotmob", "leagues"])
        _cmd_fotmob(args)
        out = capsys.readouterr().out
        assert "47" in out  # Premier League

    def test_nba_teams_handler(self, monkeypatch):
        import sportly.nba as nba
        import sportly.cli as cli
        monkeypatch.setattr(nba, "teams", lambda league_id="00": [{"id": "1610612747"}])
        args = _build_parser().parse_args(["nba", "teams"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_nba(args)
        assert "1610612747" in out.getvalue()

    def test_fantasy_meta_handler(self, monkeypatch):
        import sportly.fantasy as fantasy
        import sportly.cli as cli
        monkeypatch.setattr(fantasy, "game_meta", lambda game, cookies=None: {"currentSeasonId": 2026})
        args = _build_parser().parse_args(["fantasy", "meta", "ffl"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_fantasy(args)
        assert "2026" in out.getvalue()

    def test_fantasy_teams_handler(self, monkeypatch):
        import sportly.fantasy as fantasy
        import sportly.cli as cli
        monkeypatch.setattr(fantasy, "teams", lambda g, *, league_id, season, cookies=None: [{"id": 1}])
        args = _build_parser().parse_args(
            ["fantasy", "teams", "ffl", "--league-id", "123", "--season", "2025"]
        )
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_fantasy(args)
        assert '"id": 1' in out.getvalue()

    def test_nfl_scoreboard_handler(self, monkeypatch):
        import sportly.nfl as nfl
        import sportly.cli as cli
        monkeypatch.setattr(nfl, "scoreboard", lambda week=None, season=None, season_type=None, date=None, limit=32: [])
        args = _build_parser().parse_args(["nfl", "scoreboard", "--week", "1"])
        out = io.StringIO()
        with contextlib.redirect_stdout(out):
            cli._cmd_nfl(args)
        assert "[]" in out.getvalue()



"""sportly.mlb — MLB Stats API client.

Base URL: https://statsapi.mlb.com/api/v1/
No authentication required. v1 is the only active version.
"""

from __future__ import annotations

import httpx
from tenacity import retry, retry_if_exception_type, stop_after_attempt, wait_exponential

from sportly.exceptions import NotFoundError, RateLimitError, SportlyError

BASE_URL = "https://statsapi.mlb.com/api/v1"

# ── Sport / League IDs ────────────────────────────────────────────────────────
SPORT_ID = 1          # MLB
LEAGUE_AL = 103       # American League
LEAGUE_NL = 104       # National League

# ── Known Team IDs ────────────────────────────────────────────────────────────
TEAM_IDS: dict[str, int] = {
    "LAA": 108, "ARI": 109, "BAL": 110, "BOS": 111, "CHC": 112,
    "CIN": 113, "CLE": 114, "COL": 115, "DET": 116, "HOU": 117,
    "KC":  118, "LAD": 119, "WSH": 120, "NYM": 121, "ATH": 133,
    "PIT": 134, "SD":  135, "SEA": 136, "SF":  137, "STL": 138,
    "TB":  139, "TEX": 140, "TOR": 141, "MIN": 142, "PHI": 143,
    "ATL": 144, "CWS": 145, "MIA": 146, "NYY": 147, "MIL": 158,
}

# ── Game type codes ───────────────────────────────────────────────────────────
GAME_TYPES = {"S": "Spring Training", "R": "Regular Season", "F": "Wild Card", "D": "Division Series", "L": "LCS", "W": "World Series"}


class MLBClient:
    """Thin HTTP client for statsapi.mlb.com.

    Handles retries and error mapping. All methods return raw dicts.
    """

    def __init__(self, timeout: float = 10.0) -> None:
        self._http = httpx.Client(
            headers={"Accept": "application/json"},
            timeout=timeout,
        )

    @retry(
        retry=retry_if_exception_type(httpx.TransportError),
        stop=stop_after_attempt(3),
        wait=wait_exponential(multiplier=0.5, min=0.5, max=8),
        reraise=True,
    )
    def get(self, path: str, **params) -> dict:
        """GET {BASE_URL}/{path} with optional query params."""
        url = f"{BASE_URL}/{path.lstrip('/')}"
        try:
            resp = self._http.get(url, params={k: v for k, v in params.items() if v is not None})
        except httpx.TransportError as exc:
            raise SportlyError(str(exc)) from exc
        if resp.status_code == 404:
            raise NotFoundError(f"Not found: {url}", status_code=404, url=url)
        if resp.status_code == 429:
            raise RateLimitError("Rate limited by MLB Stats API", status_code=429)
        resp.raise_for_status()
        return resp.json()

    def close(self) -> None:
        self._http.close()

    def __enter__(self) -> "MLBClient":
        return self

    def __exit__(self, *_) -> None:
        self.close()


_default: MLBClient | None = None


def get_client() -> MLBClient:
    global _default
    if _default is None:
        _default = MLBClient()
    return _default



"""ESPN schedule/scoreboard endpoint functions."""

from __future__ import annotations

from datetime import datetime
from typing import Any

from sportly.client import SportlyClient, get_client
from sportly.espn._domains import ESPNDomain, build_url
from sportly.espn._parse import parse_game
from sportly.models import Game


def scoreboard(
    sport: str,
    league: str,
    *,
    date: str | datetime | None = None,
    limit: int = 100,
    domain: ESPNDomain = ESPNDomain.SITE,
    client: SportlyClient | None = None,
) -> list[Game]:
    """Return games for a given date (default: today).

    Parameters
    ----------
    sport:   ESPN sport slug.
    league:  League slug.
    date:    ``YYYYMMDD`` string or :class:`datetime`. Omit for today.
    limit:   Max events returned.
    domain:  ``ESPNDomain.SITE`` (default) or ``ESPNDomain.SITE_V3`` for enriched data.
    """
    http = client or get_client()
    url = build_url(domain, f"sports/{sport}/{league}/scoreboard")
    params: dict[str, Any] = {"limit": limit}
    if date:
        params["dates"] = date.strftime("%Y%m%d") if isinstance(date, datetime) else date
    data = http.get(url, params=params)
    return [parse_game(e) for e in data.get("events", [])]


def game(
    sport: str,
    league: str,
    game_id: str,
    *,
    domain: ESPNDomain = ESPNDomain.SITE,
    client: SportlyClient | None = None,
) -> Game:
    """Return the full summary for a single game.

    Parameters
    ----------
    sport:    ESPN sport slug.
    league:   League slug.
    game_id:  ESPN event ID.
    domain:   ``ESPNDomain.SITE`` (default) or ``ESPNDomain.SITE_V3``.
    """
    http = client or get_client()
    url = build_url(domain, f"sports/{sport}/{league}/summary")
    data = http.get(url, params={"event": game_id})
    return parse_game(data.get("header", {}))



"""sportly.fantasy — ESPN Fantasy API (public and private leagues).

Base URL: https://lm-api-reads.fantasy.espn.com/apis/v3/games/{gameCode}/
Only v3 on lm-api-reads is active — v1/v2 are decommissioned.

Private leagues require cookies: ``espn_s2`` + ``SWID``.

Usage::

    from sportly import fantasy

    teams = fantasy.teams("ffl", league_id=336358, season=2025)
    draft = fantasy.draft("ffl", league_id=336358, season=2025)
    meta  = fantasy.game_meta("ffl")
    print(meta["currentSeasonId"])   # 2026

    print(fantasy.GAME_CODES)   # ffl / fba / flb / fhl
    print(fantasy.VIEWS)        # all supported view names

Endpoint subpackage::

    from sportly.fantasy.endpoints import league, players, meta
"""
from __future__ import annotations
import json
from typing import Any

from sportly.fantasy.endpoints._client import FantasyClient, GAME_CODES, VIEWS

__all__ = [
    "FantasyClient", "GAME_CODES", "VIEWS",
    "_get_client",
    "league", "teams", "roster", "standings", "draft",
    "live_scoring", "transactions", "players",
    "game_meta", "season_meta",
]

BASE_URL = "https://lm-api-reads.fantasy.espn.com/apis/v3/games"

# Module-level client accessor — exposed so tests can monkeypatch it
_default: FantasyClient | None = None


def _get_client(cookies: dict[str, str] | None = None) -> FantasyClient:
    """Return or create the module-level :class:`FantasyClient`."""
    global _default
    if cookies:
        return FantasyClient(cookies=cookies)
    if _default is None:
        _default = FantasyClient()
    return _default


# ── League ────────────────────────────────────────────────────────────────────

def league(
    game_code: str,
    *,
    league_id: int,
    season: int,
    views: list[str] | None = None,
    scoring_period_id: int | None = None,
    matchup_period_id: int | None = None,
    cookies: dict[str, str] | None = None,
) -> dict[str, Any]:
    """Raw league fetch with arbitrary view(s).

    Example::

        data = fantasy.league("ffl", league_id=123, season=2025, views=["mTeam","mRoster"])
    """
    http = _get_client(cookies)
    path = f"{game_code}/seasons/{season}/segments/0/leagues/{league_id}"
    params: dict[str, Any] = {}
    if views:
        params["view"] = views
    if scoring_period_id is not None:
        params["scoringPeriodId"] = scoring_period_id
    if matchup_period_id is not None:
        params["matchupPeriodId"] = matchup_period_id
    return http.get(path, **params)  # type: ignore[return-value]


def teams(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """All fantasy teams (names, owners, records)."""
    return league(game_code, league_id=league_id, season=season, views=["mTeam"], cookies=cookies).get("teams", [])  # type: ignore[return-value]


def roster(game_code: str, *, league_id: int, season: int, scoring_period_id: int | None = None, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """All team rosters with player entries and lineup slot assignments."""
    return league(game_code, league_id=league_id, season=season, views=["mRoster"], scoring_period_id=scoring_period_id, cookies=cookies).get("teams", [])  # type: ignore[return-value]


def standings(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """Teams with W/L/PF/PA standings."""
    return league(game_code, league_id=league_id, season=season, views=["mTeam", "mStandings"], cookies=cookies).get("teams", [])  # type: ignore[return-value]


def draft(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> dict[str, Any]:
    """Full draft history (picks, player IDs, round order)."""
    return league(game_code, league_id=league_id, season=season, views=["mDraftDetail"], cookies=cookies).get("draftDetail", {})  # type: ignore[return-value]


def live_scoring(game_code: str, *, league_id: int, season: int, scoring_period_id: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """Live scoring for an active scoring period."""
    return league(game_code, league_id=league_id, season=season, views=["mBoxscore", "mLiveScoring", "mScoreboard"], scoring_period_id=scoring_period_id, cookies=cookies).get("schedule", [])  # type: ignore[return-value]


def transactions(game_code: str, *, league_id: int, season: int, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """All transactions (waivers, trades, FA adds/drops)."""
    return league(game_code, league_id=league_id, season=season, views=["mTransactions2"], cookies=cookies).get("transactions", [])  # type: ignore[return-value]


# ── Players ───────────────────────────────────────────────────────────────────

def players(game_code: str, *, season: int, active_only: bool = True, cookies: dict[str, str] | None = None) -> list[dict[str, Any]]:
    """Full player pool for a sport/season."""
    http = _get_client(cookies)
    fantasy_filter = json.dumps({"filterActive": {"value": active_only}})
    path = f"{game_code}/seasons/{season}/players"
    data = http.get(path, headers={"X-Fantasy-Filter": fantasy_filter}, view="players_wl")
    return data if isinstance(data, list) else []  # type: ignore[return-value]


# ── Metadata ──────────────────────────────────────────────────────────────────

def game_meta(game_code: str, *, cookies: dict[str, str] | None = None) -> dict[str, Any]:
    """Game metadata (``currentSeasonId``, ``name``, ``active``). No league ID required."""
    return _get_client(cookies).get(game_code)  # type: ignore[return-value]


def season_meta(game_code: str, season: int, *, cookies: dict[str, str] | None = None) -> dict[str, Any]:
    """Season-level metadata (start/end dates, current scoring period)."""
    return _get_client(cookies).get(f"{game_code}/seasons/{season}")  # type: ignore[return-value]



"""fantasy.endpoints.meta — game and season metadata."""
from __future__ import annotations
from typing import Any
from sportly.fantasy.endpoints._client import get_client


def game_meta(game_code: str, *, cookies: dict[str, str] | None = None) -> dict[str, Any]:
    """Game metadata: ``currentSeasonId``, ``name``, ``active``.

    No league ID required — works for any game code::

        from sportly.fantasy.endpoints import meta
        info = meta.game_meta("ffl")
        print(info["currentSeasonId"])   # 2026
    """
    return get_client(cookies).get(game_code)  # type: ignore[return-value]


def season_meta(game_code: str, season: int, *, cookies: dict[str, str] | None = None) -> dict[str, Any]:
    """Season-level metadata: start/end dates, current scoring period."""
    return get_client(cookies).get(f"{game_code}/seasons/{season}")  # type: ignore[return-value]



"""fotmob.endpoints.matches — daily fixture list and full match detail."""
from __future__ import annotations
from typing import Any
from sportly.fotmob.endpoints._client import FotMobClient, get_client


def matches(date: str, *, client: FotMobClient | None = None) -> dict[str, Any]:
    """All matches for a date (``YYYYMMDD``).

    Returns ``{leagues: [{id, name, matches: [{id, home, away, status}]}]}``.
    """
    return (client or get_client()).get("matches", date=date)  # type: ignore[return-value]


def match(match_id: int | str, *, client: FotMobClient | None = None) -> dict[str, Any]:
    """Full match detail: lineups, incidents, stats, xG, player ratings.

    Parameters
    ----------
    match_id: FotMob match ID (``leagues[].matches[].id`` from :func:`matches`).
    """
    return (client or get_client()).get("matchDetails", matchId=match_id)  # type: ignore[return-value]



"""Pydantic data models returned by all sportly methods.

All models use ``model_config = ConfigDict(extra="allow")`` so that
new fields added by upstream APIs are preserved rather than dropped.
"""

from __future__ import annotations

from datetime import datetime
from typing import Any

from pydantic import BaseModel, ConfigDict, Field


class _Base(BaseModel):
    """Shared config for all sportly models."""

    model_config = ConfigDict(extra="allow", populate_by_name=True)


# ── Teams ─────────────────────────────────────────────────────────────────────


class Logo(BaseModel):
    model_config = ConfigDict(extra="allow")
    href: str = ""
    width: int | None = None
    height: int | None = None
    rel: list[str] = Field(default_factory=list)


class Team(_Base):
    """A sports team.

    Attributes
    ----------
    id:       ESPN team ID (string).
    uid:      Full ESPN UID, e.g. ``"s:40~l:46~t:13"``.
    slug:     URL-safe slug, e.g. ``"los-angeles-lakers"``.
    abbr:     Short abbreviation, e.g. ``"LAL"``.
    name:     Full display name, e.g. ``"Los Angeles Lakers"``.
    nickname: Short name, e.g. ``"Lakers"``.
    location: City / region, e.g. ``"Los Angeles"``.
    color:    Primary hex colour (no ``#``).
    alt_color: Alternate hex colour.
    is_active: Whether the team is currently active.
    logos:    List of logo objects.
    raw:      Full raw dict from the upstream API.
    """

    id: str = ""
    uid: str = ""
    slug: str = ""
    abbr: str = Field("", alias="abbreviation")
    name: str = Field("", alias="displayName")
    nickname: str = ""
    location: str = ""
    color: str = ""
    alt_color: str = Field("", alias="alternateColor")
    is_active: bool = Field(True, alias="isActive")
    logos: list[Logo] = Field(default_factory=list)
    raw: dict[str, Any] = Field(default_factory=dict)


# ── Events / Games ────────────────────────────────────────────────────────────


class Competitor(_Base):
    """One side of a game (home or away)."""

    id: str = ""
    team: Team | None = None
    home_away: str = Field("", alias="homeAway")
    score: str = ""
    winner: bool | None = None


class GameStatus(_Base):
    """Current game status."""

    state: str = ""       # "pre" | "in" | "post"
    detail: str = ""      # e.g. "Final", "Q3 4:23", "7:30 PM ET"
    completed: bool = False
    period: int | None = None
    clock: str = ""


class Game(_Base):
    """A single game / event.

    Attributes
    ----------
    id:          ESPN event ID.
    name:        Long game name, e.g. ``"Los Angeles Lakers vs Boston Celtics"``.
    short_name:  Short form, e.g. ``"LAL vs BOS"``.
    date:        UTC tip-off / kick-off time.
    status:      Current game status.
    competitors: Home and away team info including scores.
    raw:         Full raw dict from the upstream API.
    """

    id: str = ""
    uid: str = ""
    name: str = ""
    short_name: str = Field("", alias="shortName")
    date: datetime | None = None
    status: GameStatus = Field(default_factory=GameStatus)
    competitors: list[Competitor] = Field(default_factory=list)
    raw: dict[str, Any] = Field(default_factory=dict)


# ── News ─────────────────────────────────────────────────────────────────────


class Article(_Base):
    """A news article."""

    id: str = ""
    headline: str = ""
    description: str = ""
    published: datetime | None = None
    type: str = ""
    images: list[dict[str, Any]] = Field(default_factory=list)
    links: dict[str, Any] = Field(default_factory=dict)

    @property
    def thumbnail(self) -> str | None:
        """Return the first image URL, if any."""
        for img in self.images:
            return img.get("url") or img.get("href")
        return None


# ── Standings ─────────────────────────────────────────────────────────────────


class StandingEntry(_Base):
    """One team row in the standings table."""

    team: Team | None = None
    wins: int = 0
    losses: int = 0
    win_pct: float = Field(0.0, alias="winPercent")
    games_behind: float | None = Field(None, alias="gamesBehind")
    raw: dict[str, Any] = Field(default_factory=dict)


class Standings(_Base):
    """League standings."""

    name: str = ""
    season: int | None = None
    entries: list[StandingEntry] = Field(default_factory=list)
    raw: dict[str, Any] = Field(default_factory=dict)


# ── Athlete ───────────────────────────────────────────────────────────────────


class Athlete(_Base):
    """An athlete / player."""

    id: str = ""
    full_name: str = Field("", alias="fullName")
    display_name: str = Field("", alias="displayName")
    short_name: str = Field("", alias="shortName")
    position: str = ""
    jersey: str = ""
    team: Team | None = None
    headshot: str = ""
    is_active: bool = Field(True, alias="active")
    raw: dict[str, Any] = Field(default_factory=dict)



"""sofascore.endpoints.tournaments — competitions and popular leagues."""
from __future__ import annotations
from typing import Any
from sportly.sofascore.endpoints._client import SPORTS, get


def tournaments(sport: str, *, client=None) -> dict[str, Any]:
    """All available tournaments / unique leagues for a sport."""
    slug = SPORTS.get(sport, sport)
    return get(f"sport/{slug}/unique-tournaments", client=client)  # type: ignore[return-value]


def popular(locale: str = "US", *, client=None) -> dict[str, Any]:
    """Popular leagues and entities for a locale (e.g. ``"US"``, ``"GB"``)."""
    return get(f"config/popular-entities/{locale}", client=client)  # type: ignore[return-value]



"""sportly.mlb.endpoints.teams — Team and roster endpoints."""
from __future__ import annotations
from typing import Any
from sportly.mlb._client import MLBClient, SPORT_ID, get_client


def all(
    *,
    sport_id: int = SPORT_ID,
    season: int | None = None,
    league_ids: str | None = None,
    fields: str | None = None,
    client: MLBClient | None = None,
) -> list[dict[str, Any]]:
    """Return all MLB teams.

    Parameters
    ----------
    sport_id:   ``1`` = MLB (default), ``11`` = AAA, ``12`` = AA.
    season:     Filter to teams active in a specific year.
    league_ids: ``"103,104"`` for AL+NL.
    fields:     Comma-separated field list to trim response.
    """
    http = client or get_client()
    data = http.get("teams", sportId=sport_id, season=season, leagueIds=league_ids, fields=fields)
    return data.get("teams", [])  # type: ignore[return-value]


def one(
    team_id: int | str,
    *,
    season: int | None = None,
    fields: str | None = None,
    client: MLBClient | None = None,
) -> dict[str, Any]:
    """Return a single MLB team by ID (e.g. ``119`` = Dodgers)."""
    http = client or get_client()
    data = http.get(f"teams/{team_id}", season=season, fields=fields)
    teams = data.get("teams", [{}])
    return teams[0] if teams else {}  # type: ignore[return-value]


def roster(
    team_id: int | str,
    *,
    roster_type: str = "active",
    season: int | None = None,
    client: MLBClient | None = None,
) -> list[dict[str, Any]]:
    """Return the roster for a team.

    Parameters
    ----------
    roster_type: ``"active"`` (default), ``"40Man"``, ``"fullRoster"``.
    """
    http = client or get_client()
    data = http.get(f"teams/{team_id}/roster", rosterType=roster_type, season=season)
    return data.get("roster", [])  # type: ignore[return-value]


def stats(
    team_id: int | str,
    *,
    stats: str = "season",
    group: str = "hitting",
    season: int | None = None,
    client: MLBClient | None = None,
) -> list[dict[str, Any]]:
    """Return team stats.

    Parameters
    ----------
    stats: ``"season"``, ``"career"``, ``"yearByYear"``.
    group: ``"hitting"``, ``"pitching"``, ``"fielding"``.
    """
    http = client or get_client()
    data = http.get(f"teams/{team_id}/stats", stats=stats, group=group, season=season)
    return data.get("stats", [])  # type: ignore[return-value]



"""sportly CLI entry point.

Installed as the ``sportly`` command via ``[project.scripts]`` in pyproject.toml.

Usage::

    sportly info

    # ESPN
    sportly espn basketball teams --league nba
    sportly espn football scoreboard --league nfl
    sportly espn soccer standings --league eng.1

    # Native APIs
    sportly nhl teams
    sportly nhl schedule --date 2024-04-15

    sportly mlb schedule
    sportly mlb schedule --date 2025-04-01
    sportly mlb teams
    sportly mlb player 660271
    sportly mlb leaders homeRuns --season 2025

    sportly nba scoreboard 2025-03-26
    sportly nba leaders PTS --season 2024-25
    sportly nba player 2544

    sportly nfl scoreboard --week 1 --season 2024
    sportly nfl teams
    sportly nfl injuries

    sportly fotmob matches 20260326
    sportly fotmob league 47
    sportly fotmob team 8456
    sportly fotmob search "erling haaland"
"""

from __future__ import annotations

import argparse
import json
import sys


def _dump(obj: object) -> None:
    """Print a result as pretty JSON."""
    if hasattr(obj, "model_dump"):
        print(json.dumps(obj.model_dump(mode="json"), indent=2, default=str))
    elif isinstance(obj, list):
        if obj and hasattr(obj[0], "model_dump"):
            print(json.dumps([o.model_dump(mode="json") for o in obj], indent=2, default=str))
        else:
            print(json.dumps(obj, indent=2, default=str))
    else:
        print(json.dumps(obj, indent=2, default=str))


# ── info ──────────────────────────────────────────────────────────────────────

def _cmd_info(args: argparse.Namespace) -> None:  # noqa: ARG001
    from sportly import __version__, __author__, __email__
    print(f"sportly {__version__}")
    print(f"Author : {__author__} <{__email__}>")
    print("Repo   : https://github.com/pseudo-r/sportly")
    print()
    print("Available namespaces:")
    namespaces = [
        ("sportly.espn", "17 sports, 139+ leagues"),
        ("sportly.nhl",  "NHL Web API + Stats REST"),
        ("sportly.mlb",  "MLB Stats API"),
        ("sportly.nba",  "NBA Stats API (WAF headers auto-injected)"),
        ("sportly.nfl",  "NFL via ESPN public infrastructure"),
        ("sportly.fantasy", "ESPN Fantasy v3 (ffl/fba/flb/fhl)"),
        ("sportly.fotmob",  "FotMob web API (xG, ratings, shot maps)"),
        ("sportly.sofascore", "Sofascore API — requires pip install sportly[sofascore]"),
    ]
    for ns, desc in namespaces:
        print(f"  {ns:<28} {desc}")


# ── ESPN ──────────────────────────────────────────────────────────────────────

_ESPN_SPORTS = {
    "basketball", "football", "soccer", "hockey", "baseball", "cricket",
    "tennis", "golf", "mma", "racing", "lacrosse", "volleyball",
    "water_polo", "field_hockey", "rugby", "rugby_league", "australian_football",
}

_ESPN_METHODS = {
    "teams", "team", "roster", "scoreboard", "game",
    "news", "standings", "rankings", "athlete", "injuries",
    "transactions", "leaders", "play_by_play", "odds",
}


def _cmd_espn(args: argparse.Namespace) -> None:
    sport  = args.sport
    method = args.method

    if sport not in _ESPN_SPORTS:
        print(f"Unknown sport '{sport}'. Choose from: {', '.join(sorted(_ESPN_SPORTS))}")
        sys.exit(1)

    if method not in _ESPN_METHODS:
        print(f"Unknown method '{method}'. Choose from: {', '.join(sorted(_ESPN_METHODS))}")
        sys.exit(1)

    import importlib
    mod = importlib.import_module(f"sportly.espn.{sport}")
    fn = getattr(mod, method, None)
    if fn is None:
        print(f"Method '{method}' not available for '{sport}'.")
        sys.exit(1)

    kwargs: dict[str, object] = {}
    if hasattr(args, "league") and args.league:
        kwargs["league"] = args.league
    if hasattr(args, "date") and args.date:
        kwargs["date"] = args.date

    if hasattr(args, "id") and args.id:
        result = fn(args.id, **kwargs)
    else:
        result = fn(**kwargs)
    _dump(result)


# ── NHL ───────────────────────────────────────────────────────────────────────

def _cmd_nhl(args: argparse.Namespace) -> None:
    from sportly.nhl import _inst as nhl
    if args.nhl_cmd == "teams":
        _dump(nhl.teams())
    elif args.nhl_cmd == "schedule":
        _dump(nhl.schedule(getattr(args, "date", None)))
    elif args.nhl_cmd == "game":
        _dump(nhl.game(args.id))
    elif args.nhl_cmd == "play-by-play":
        _dump(nhl.play_by_play(args.id))
    else:
        print(f"Unknown NHL command '{args.nhl_cmd}'.")
        sys.exit(1)


# ── MLB ───────────────────────────────────────────────────────────────────────

def _cmd_mlb(args: argparse.Namespace) -> None:
    import sportly.mlb as mlb
    cmd = args.mlb_cmd

    if cmd == "schedule":
        _dump(mlb.schedule(date=getattr(args, "date", None)))
    elif cmd == "teams":
        _dump(mlb.teams())
    elif cmd == "team":
        _dump(mlb.team(args.id))
    elif cmd == "roster":
        _dump(mlb.roster(args.id))
    elif cmd == "player":
        _dump(mlb.player(args.id))
    elif cmd == "boxscore":
        _dump(mlb.boxscore(args.id))
    elif cmd == "linescore":
        _dump(mlb.linescore(args.id))
    elif cmd == "play-by-play":
        _dump(mlb.play_by_play(args.id))
    elif cmd == "standings":
        _dump(mlb.standings(season=getattr(args, "season", None)))
    elif cmd == "leaders":
        _dump(mlb.leaders(args.category, season=getattr(args, "season", None)))
    elif cmd == "transactions":
        _dump(mlb.transactions(start_date=args.start_date, end_date=args.end_date))
    else:
        print(f"Unknown MLB command '{cmd}'.")
        sys.exit(1)


# ── NBA ───────────────────────────────────────────────────────────────────────

def _cmd_nba(args: argparse.Namespace) -> None:
    import sportly.nba as nba
    cmd = args.nba_cmd

    if cmd == "scoreboard":
        _dump(nba.scoreboard(args.date))
    elif cmd == "teams":
        _dump(nba.teams())
    elif cmd == "player":
        _dump(nba.player(args.id))
    elif cmd == "career":
        _dump(nba.career_stats(args.id))
    elif cmd == "leaders":
        season = getattr(args, "season", "2024-25") or "2024-25"
        _dump(nba.leaders(args.stat, season=season))
    elif cmd == "standings":
        season = getattr(args, "season", "2024-25") or "2024-25"
        _dump(nba.standings(season))
    elif cmd == "shot-chart":
        season = getattr(args, "season", "2024-25") or "2024-25"
        _dump(nba.shot_chart(args.id, season))
    else:
        print(f"Unknown NBA command '{cmd}'.")
        sys.exit(1)


# ── NFL ───────────────────────────────────────────────────────────────────────

def _cmd_nfl(args: argparse.Namespace) -> None:
    import sportly.nfl as nfl
    cmd = args.nfl_cmd

    if cmd == "scoreboard":
        week   = getattr(args, "week", None)
        season = getattr(args, "season", None)
        _dump(nfl.scoreboard(week=week, season=season))
    elif cmd == "teams":
        _dump(nfl.teams())
    elif cmd == "team":
        _dump(nfl.team(args.id))
    elif cmd == "injuries":
        _dump(nfl.injuries())
    elif cmd == "news":
        _dump(nfl.news())
    elif cmd == "standings":
        _dump(nfl.standings())
    elif cmd == "play-by-play":
        _dump(nfl.play_by_play(args.id))
    elif cmd == "depth-chart":
        _dump(nfl.depth_chart(args.id))
    elif cmd == "qbr":
        season = getattr(args, "season", None)
        _dump(nfl.qbr(season=season))
    else:
        print(f"Unknown NFL command '{cmd}'.")
        sys.exit(1)


# ── FotMob ────────────────────────────────────────────────────────────────────

def _cmd_fotmob(args: argparse.Namespace) -> None:
    import sportly.fotmob as fotmob
    cmd = args.fm_cmd

    if cmd == "matches":
        _dump(fotmob.matches(args.date))
    elif cmd == "match":
        _dump(fotmob.match(args.id))
    elif cmd == "league":
        _dump(fotmob.league(args.id))
    elif cmd == "team":
        _dump(fotmob.team(args.id))
    elif cmd == "player":
        _dump(fotmob.player(args.id))
    elif cmd == "search":
        _dump(fotmob.search(args.term))
    elif cmd == "leagues":
        _dump(fotmob.LEAGUES)
    else:
        print(f"Unknown FotMob command '{cmd}'.")
        sys.exit(1)


# ── Fantasy ───────────────────────────────────────────────────────────────────

def _cmd_fantasy(args: argparse.Namespace) -> None:
    import sportly.fantasy as fantasy
    cmd     = args.fs_cmd
    game    = args.game
    lid     = getattr(args, "league_id", None)
    season  = getattr(args, "season", None)
    cookies = None
    if getattr(args, "espn_s2", None) and getattr(args, "swid", None):
        cookies = {"espn_s2": args.espn_s2, "SWID": args.swid}

    if cmd == "league":
        views = args.views.split(",") if getattr(args, "views", None) else None
        _dump(fantasy.league(game, league_id=lid, season=season, views=views, cookies=cookies))
    elif cmd == "teams":
        _dump(fantasy.teams(game, league_id=lid, season=season, cookies=cookies))
    elif cmd == "roster":
        _dump(fantasy.roster(game, league_id=lid, season=season, cookies=cookies))
    elif cmd == "standings":
        _dump(fantasy.standings(game, league_id=lid, season=season, cookies=cookies))
    elif cmd == "draft":
        _dump(fantasy.draft(game, league_id=lid, season=season, cookies=cookies))
    elif cmd == "meta":
        _dump(fantasy.game_meta(game, cookies=cookies))
    else:
        print(f"Unknown Fantasy command '{cmd}'.")
        sys.exit(1)


# ── Sofascore ─────────────────────────────────────────────────────────────────

def _cmd_sofascore(args: argparse.Namespace) -> None:
    try:
        import sportly.sofascore as sofascore
    except ImportError:
        print("sportly.sofascore requires curl_cffi. Install with: pip install sportly[sofascore]")
        sys.exit(1)
    cmd = args.ss_cmd

    if cmd == "matches":
        _dump(sofascore.matches(args.sport, args.date))
    elif cmd == "match":
        _dump(sofascore.match(args.id))
    elif cmd == "stats":
        _dump(sofascore.match_stats(args.id))
    elif cmd == "lineups":
        _dump(sofascore.lineups(args.id))
    elif cmd == "player":
        _dump(sofascore.player(args.id))
    elif cmd == "team":
        _dump(sofascore.team(args.id))
    elif cmd == "squad":
        _dump(sofascore.squad(args.id))
    else:
        print(f"Unknown Sofascore command '{cmd}'.")
        sys.exit(1)


# ── Argument parser ───────────────────────────────────────────────────────────

def _build_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="sportly",
        description="Python SDK CLI for multi-source sports data",
    )
    sub = parser.add_subparsers(dest="command")

    # ── info ──────────────────────────────────────────────────────────────────
    sub.add_parser("info", help="Show version and available namespaces")

    # ── espn ──────────────────────────────────────────────────────────────────
    espn_p = sub.add_parser("espn", help="ESPN data (17 sports)")
    espn_p.add_argument("sport",  choices=sorted(_ESPN_SPORTS), help="Sport slug")
    espn_p.add_argument("method", choices=sorted(_ESPN_METHODS), help="Method to call")
    espn_p.add_argument("--league", "-l", default=None, help="League slug, e.g. nba")
    espn_p.add_argument("--date",   "-d", default=None, help="Date YYYYMMDD")
    espn_p.add_argument("--id",           default=None, help="Team / game / athlete ID")

    # ── nhl ───────────────────────────────────────────────────────────────────
    nhl_p   = sub.add_parser("nhl", help="Native NHL API")
    nhl_sub = nhl_p.add_subparsers(dest="nhl_cmd")
    nhl_sub.add_parser("teams", help="All NHL franchises")
    sp = nhl_sub.add_parser("schedule", help="Schedule for a date")
    sp.add_argument("--date", "-d", default=None, help="Date YYYY-MM-DD")
    gp = nhl_sub.add_parser("game", help="Game boxscore")
    gp.add_argument("id", help="NHL game ID")
    pp = nhl_sub.add_parser("play-by-play", help="Play-by-play")
    pp.add_argument("id", help="NHL game ID")

    # ── mlb ───────────────────────────────────────────────────────────────────
    mlb_p   = sub.add_parser("mlb", help="MLB Stats API")
    mlb_sub = mlb_p.add_subparsers(dest="mlb_cmd")
    mlb_sub.add_parser("teams", help="All MLB teams")
    sched = mlb_sub.add_parser("schedule", help="Games for a date")
    sched.add_argument("--date", "-d", default=None, help="Date YYYY-MM-DD")
    for cmd in ("team", "roster", "player", "boxscore", "linescore", "play-by-play"):
        p2 = mlb_sub.add_parser(cmd, help=f"MLB {cmd}")
        p2.add_argument("id", help="Team / game / player ID")
    st = mlb_sub.add_parser("standings", help="Division standings")
    st.add_argument("--season", default=None, type=int, help="Season year")
    ld = mlb_sub.add_parser("leaders", help="Statistical leaders")
    ld.add_argument("category", help="E.g. homeRuns, era, battingAverage")
    ld.add_argument("--season", default=None, type=int, help="Season year")
    tx = mlb_sub.add_parser("transactions", help="IL moves, trades")
    tx.add_argument("--start-date", required=True, dest="start_date", help="YYYY-MM-DD")
    tx.add_argument("--end-date",   required=True, dest="end_date",   help="YYYY-MM-DD")

    # ── nba ───────────────────────────────────────────────────────────────────
    nba_p   = sub.add_parser("nba", help="NBA Stats API")
    nba_sub = nba_p.add_subparsers(dest="nba_cmd")
    nba_sub.add_parser("teams", help="All NBA teams")
    sc = nba_sub.add_parser("scoreboard", help="Scoreboard for a date")
    sc.add_argument("date", help="Date YYYY-MM-DD")
    for cmd in ("player", "career", "shot-chart"):
        p2 = nba_sub.add_parser(cmd, help=f"NBA {cmd}")
        p2.add_argument("id", help="NBA player ID")
        if cmd in ("career", "shot-chart"):
            p2.add_argument("--season", default="2024-25", help="Season e.g. 2024-25")
    ld2 = nba_sub.add_parser("leaders", help="League leaders")
    ld2.add_argument("stat", help="E.g. PTS, REB, AST")
    ld2.add_argument("--season", default="2024-25", help="Season e.g. 2024-25")
    st2 = nba_sub.add_parser("standings", help="League standings")
    st2.add_argument("--season", default="2024-25", help="Season e.g. 2024-25")

    # ── nfl ───────────────────────────────────────────────────────────────────
    nfl_p   = sub.add_parser("nfl", help="NFL via ESPN")
    nfl_sub = nfl_p.add_subparsers(dest="nfl_cmd")
    for cmd in ("teams", "injuries", "news", "standings", "qbr"):
        nfl_sub.add_parser(cmd, help=f"NFL {cmd}")
    sb_nfl = nfl_sub.add_parser("scoreboard", help="NFL scoreboard")
    sb_nfl.add_argument("--week",   type=int, default=None, help="NFL week number")
    sb_nfl.add_argument("--season", type=int, default=None, help="Season year")
    for cmd in ("team", "depth-chart", "play-by-play"):
        p2 = nfl_sub.add_parser(cmd, help=f"NFL {cmd}")
        p2.add_argument("id", help="Team / game ID")

    # ── fotmob ────────────────────────────────────────────────────────────────
    fm_p   = sub.add_parser("fotmob", help="FotMob football data")
    fm_sub = fm_p.add_subparsers(dest="fm_cmd")
    fm_sub.add_parser("leagues", help="Show all popular league IDs")
    mt = fm_sub.add_parser("matches", help="All matches for a date")
    mt.add_argument("date", help="Date YYYYMMDD e.g. 20260326")
    sr = fm_sub.add_parser("search", help="Search players/teams/tournaments")
    sr.add_argument("term", help="Search term")
    for cmd in ("match", "league", "team", "player"):
        p2 = fm_sub.add_parser(cmd, help=f"FotMob {cmd} by ID")
        p2.add_argument("id", type=int, help="Numeric FotMob ID")

    # ── fantasy ───────────────────────────────────────────────────────────────
    def _add_fantasy_common(p: argparse.ArgumentParser) -> None:
        p.add_argument("game", choices=["ffl", "fba", "flb", "fhl"], help="Game code")
        p.add_argument("--league-id", required=True, type=int, dest="league_id", help="League ID")
        p.add_argument("--season",    required=True, type=int, help="Season year")
        p.add_argument("--espn-s2",   dest="espn_s2", default=None, help="espn_s2 cookie (private leagues)")
        p.add_argument("--swid",      dest="swid",    default=None, help="SWID cookie (private leagues)")

    fs_p   = sub.add_parser("fantasy", help="ESPN Fantasy (ffl/fba/flb/fhl)")
    fs_sub = fs_p.add_subparsers(dest="fs_cmd")
    for cmd in ("teams", "roster", "standings", "draft"):
        p2 = fs_sub.add_parser(cmd, help=f"Fantasy {cmd}")
        _add_fantasy_common(p2)
    lg_p = fs_sub.add_parser("league", help="Raw league data by view")
    _add_fantasy_common(lg_p)
    lg_p.add_argument("--views", default=None, help="Comma-separated views e.g. mTeam,mRoster")
    mt_p = fs_sub.add_parser("meta", help="Game metadata")
    mt_p.add_argument("game", choices=["ffl", "fba", "flb", "fhl"], help="Game code")
    mt_p.add_argument("--espn-s2", dest="espn_s2", default=None)
    mt_p.add_argument("--swid",    dest="swid",    default=None)

    # ── sofascore ─────────────────────────────────────────────────────────────
    ss_p   = sub.add_parser("sofascore", help="Sofascore (requires pip install sportly[sofascore])")
    ss_sub = ss_p.add_subparsers(dest="ss_cmd")
    ss_mt = ss_sub.add_parser("matches", help="Events for sport/date")
    ss_mt.add_argument("sport", help="Sport slug e.g. football, basketball, tennis")
    ss_mt.add_argument("date",  help="Date YYYY-MM-DD")
    for cmd in ("match", "stats", "lineups"):
        p2 = ss_sub.add_parser(cmd, help=f"Sofascore {cmd} by event ID")
        p2.add_argument("id", type=int, help="Event / match ID")
    for cmd in ("player", "team", "squad"):
        p2 = ss_sub.add_parser(cmd, help=f"Sofascore {cmd} by ID")
        p2.add_argument("id", type=int, help="Player / team ID")

    return parser


def main() -> None:
    """Entry point for the ``sportly`` CLI command."""
    parser = _build_parser()
    args = parser.parse_args()

    dispatch = {
        "info":       _cmd_info,
        "espn":       _cmd_espn,
        "nhl":        _cmd_nhl,
        "mlb":        _cmd_mlb,
        "nba":        _cmd_nba,
        "nfl":        _cmd_nfl,
        "fotmob":     _cmd_fotmob,
        "fantasy":    _cmd_fantasy,
        "sofascore":  _cmd_sofascore,
    }

    handler = dispatch.get(args.command)
    if handler:
        handler(args)
    else:
        parser.print_help()
        sys.exit(0)


if __name__ == "__main__":
    main()



"""sportly — Python SDK for multi-source sports data.

Sources
-------
- ``sportly.espn``      ESPN (17 sports, 139+ leagues)
- ``sportly.nhl``       NHL Web API + Stats REST
- ``sportly.mlb``       MLB Stats API
- ``sportly.nba``       NBA Stats API (WAF headers auto-injected)
- ``sportly.nfl``       NFL via ESPN public infrastructure
- ``sportly.fantasy``   ESPN Fantasy API v3 (public + private leagues)
- ``sportly.fotmob``    FotMob web API (xG, shot maps, ratings)
- ``sportly.sofascore`` Sofascore API (requires ``pip install sportly[sofascore]``)

Quick start
-----------
::

    from sportly import mlb, nba, nfl, fotmob

    # Today's MLB games
    mlb.schedule()

    # NBA scoring leaders
    nba.leaders("PTS", season="2024-25")

    # NFL scoreboard, Week 1
    nfl.scoreboard(week=1, season=2024)

    # Premier League table
    fotmob.league(47)

    # ESPN fantasy roster
    from sportly import fantasy
    fantasy.roster("ffl", league_id=123456, season=2025)

"""

__version__ = "1.1.0"
__author__ = "Joseph Wilson"
__email__ = "jwilson@kloverdevs.ca"

# Core client and exceptions — importable at top level
from sportly.client import SportlyClient  # noqa: F401
from sportly.exceptions import NotFoundError, RateLimitError, SportlyError  # noqa: F401

__all__ = [
    "__version__",
    "SportlyClient",
    "SportlyError",
    "NotFoundError",
    "RateLimitError",
]



"""Tests for sportly.sofascore — mocked without curl_cffi (patches _get)."""
from __future__ import annotations
import pytest


# ── Sofascore tests use monkeypatching on the internal _get function ──────────
# because curl_cffi isn't installed in the test env (it's an optional extra).

MATCH_BODY  = {"event": {"id": 11352523, "tournament": {"name": "World Cup"}, "homeScore": {"current": 2}}}
STATS_BODY  = {"statistics": [{"period": "ALL", "groups": [{"groupName": "Football", "statisticsItems": []}]}]}
LINEUP_BODY = {"home": {"players": [{"player": {"name": "Ronaldo"}}]}, "away": {"players": []}}
PLAYER_BODY = {"player": {"id": 814123, "name": "Erling Haaland", "team": {"name": "Manchester City"}}}
TEAM_BODY   = {"team": {"id": 4705, "name": "Juventus"}}
SQUAD_BODY  = {"players": [{"player": {"id": 814123, "name": "Erling Haaland"}}]}
EVENTS_BODY = {"events": [{"id": 11352523, "homeTeam": {"name": "France"}, "awayTeam": {"name": "Germany"}}]}
TOURS_BODY  = {"uniqueTournaments": [{"id": 47, "name": "Premier League"}]}


class TestSofascoreConstants:
    def test_sports_slugs(self):
        from sportly.sofascore import SPORTS
        assert "football" in SPORTS
        assert SPORTS["soccer"] == "football"
        assert SPORTS["hockey"] == "ice-hockey"


class TestSofascoreEndpoints:
    def _patch(self, monkeypatch, body):
        import sportly.sofascore as ss
        monkeypatch.setattr(ss, "_get", lambda path, client=None: body)

    def test_matches_football(self, monkeypatch):
        import sportly.sofascore as ss
        monkeypatch.setattr(ss, "_get", lambda path, client=None: EVENTS_BODY)
        result = ss.matches("football", "2026-03-26")
        assert result[0]["homeTeam"]["name"] == "France"

    def test_match_detail(self, monkeypatch):
        self._patch(monkeypatch, MATCH_BODY)
        import sportly.sofascore as ss
        m = ss.match(11352523)
        assert m["event"]["id"] == 11352523

    def test_match_stats(self, monkeypatch):
        self._patch(monkeypatch, STATS_BODY)
        import sportly.sofascore as ss
        s = ss.match_stats(11352523)
        assert "statistics" in s

    def test_lineups(self, monkeypatch):
        self._patch(monkeypatch, LINEUP_BODY)
        import sportly.sofascore as ss
        l = ss.lineups(11352523)
        assert l["home"]["players"][0]["player"]["name"] == "Ronaldo"

    def test_player(self, monkeypatch):
        self._patch(monkeypatch, PLAYER_BODY)
        import sportly.sofascore as ss
        p = ss.player(814123)
        assert p["player"]["name"] == "Erling Haaland"

    def test_team(self, monkeypatch):
        self._patch(monkeypatch, TEAM_BODY)
        import sportly.sofascore as ss
        t = ss.team(4705)
        assert t["team"]["name"] == "Juventus"

    def test_squad_extracts_players(self, monkeypatch):
        self._patch(monkeypatch, SQUAD_BODY)
        import sportly.sofascore as ss
        players = ss.squad(4705)
        assert players[0]["player"]["name"] == "Erling Haaland"

    def test_tournaments(self, monkeypatch):
        self._patch(monkeypatch, TOURS_BODY)
        import sportly.sofascore as ss
        t = ss.tournaments("football")
        assert "uniqueTournaments" in t

    def test_callable_api(self):
        import sportly.sofascore as ss
        assert callable(ss.matches)
        assert callable(ss.match)
        assert callable(ss.player)
        assert callable(ss.squad)
        assert callable(ss.momentum)
        assert callable(ss.popular)

    def test_missing_curl_cffi_raises_import_error(self):
        """Without curl_cffi, _get_client() raises ImportError with helpful message."""
        import importlib, sys
        # Temporarily hide curl_cffi
        saved = sys.modules.get("curl_cffi")
        sys.modules["curl_cffi"] = None  # type: ignore
        try:
            import sportly.sofascore as ss
            with pytest.raises(ImportError, match="curl_cffi"):
                ss._get_client()
        finally:
            if saved is None:
                sys.modules.pop("curl_cffi", None)
            else:
                sys.modules["curl_cffi"] = saved



"""sportly.espn.football — NFL, college football, CFL."""
from __future__ import annotations
from typing import Any
from sportly.client import SportlyClient, get_client
from sportly.espn._domains import ESPNDomain, build_url
from sportly.espn._leagues import LEAGUES as _ALL_LEAGUES
from sportly.espn.endpoints import advanced as _advanced_ep
from sportly.espn.endpoints import athletes as _athletes_ep
from sportly.espn.endpoints import news as _news_ep
from sportly.espn.endpoints import schedule as _schedule_ep
from sportly.espn.endpoints import standings as _standings_ep
from sportly.espn.endpoints import teams as _teams_ep
from sportly.models import Article, Athlete, Game, Standings, Team

SPORT = "football"
DEFAULT_LEAGUE = "nfl"
LEAGUES: dict[str, str] = _ALL_LEAGUES[SPORT]

def teams(league: str = DEFAULT_LEAGUE, *, limit: int = 100, client: SportlyClient | None = None) -> list[Team]:
    return _teams_ep.all(SPORT, league, limit=limit, client=client)

def team(team_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Team:
    return _teams_ep.one(SPORT, league, team_id, client=client)

def roster(team_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[Athlete]:
    return _teams_ep.roster(SPORT, league, team_id, client=client)

def scoreboard(league: str = DEFAULT_LEAGUE, *, date: str | None = None, limit: int = 100, domain: ESPNDomain = ESPNDomain.SITE, client: SportlyClient | None = None) -> list[Game]:
    return _schedule_ep.scoreboard(SPORT, league, date=date, limit=limit, domain=domain, client=client)

def game(game_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Game:
    return _schedule_ep.game(SPORT, league, game_id, client=client)

def news(league: str = DEFAULT_LEAGUE, *, limit: int = 25, client: SportlyClient | None = None) -> list[Article]:
    return _news_ep.news(SPORT, league, limit=limit, client=client)

def standings(league: str = DEFAULT_LEAGUE, *, season: int | None = None, client: SportlyClient | None = None) -> Standings:
    return _standings_ep.standings(SPORT, league, season=season, client=client)

def rankings(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    return _standings_ep.rankings(SPORT, league, client=client)

def injuries(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    return _advanced_ep.injuries(SPORT, league, client=client)

def transactions(league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> list[dict[str, Any]]:
    return _advanced_ep.transactions(SPORT, league, client=client)

def athlete(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> Athlete:
    return _athletes_ep.athlete(SPORT, league, athlete_id, client=client)

def athlete_stats(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    return _athletes_ep.stats(SPORT, league, athlete_id, client=client)

def athlete_gamelog(athlete_id: str | int, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    return _athletes_ep.gamelog(SPORT, league, athlete_id, client=client)

def odds(event_id: str, league: str = DEFAULT_LEAGUE, *, client: SportlyClient | None = None) -> dict[str, Any]:
    return _advanced_ep.odds(SPORT, league, event_id, client=client)

def play_by_play(event_id: str, league: str = DEFAULT_LEAGUE, *, limit: int = 300, client: SportlyClient | None = None) -> dict[str, Any]:
    return _advanced_ep.play_by_play(SPORT, league, event_id, limit=limit, client=client)

def cdn_game(game_id: str, *, client: SportlyClient | None = None) -> dict[str, Any]:
    return _advanced_ep.cdn_game(SPORT, game_id, client=client)

# ── Football-specific ─────────────────────────────────────────────────────────

def qbr(league: str = DEFAULT_LEAGUE, *, season: int | None = None, season_type: int = 2, week: int | None = None, split: int = 0, client: SportlyClient | None = None) -> dict[str, Any]:
    """ESPN Quarterback Rating (QBR). ``season_type``: 2=regular(default), 1=pre, 3=post."""
    http = client or get_client()
    year = season or "current"
    base = f"sports/{SPORT}/leagues/{league}/seasons/{year}/types/{season_type}"
    path = f"{base}/weeks/{week}/qbr/{split}" if week else f"{base}/groups/1/qbr/{split}"
    return http.get(build_url(ESPNDomain.CORE, path))  # type: ignore[return-value]

def power_index(league: str = "college-football", *, season: int | None = None, client: SportlyClient | None = None) -> dict[str, Any]:
    """SP+ power ratings (college) or NFL power index."""
    http = client or get_client()
    base = f"sports/{SPORT}/leagues/{league}"
    path = f"{base}/seasons/{season}/powerindex" if season else f"{base}/powerindex"
    return http.get(build_url(ESPNDomain.CORE, path))  # type: ignore[return-value]



"""sportly.nba.endpoints.league — Standings, leaders, draft."""
from __future__ import annotations
from typing import Any
from sportly.nba._client import NBAClient, LEAGUE_NBA, parse_result_sets, get_client


def standings(
    season: str,
    *,
    season_type: str = "Regular Season",
    league_id: str = LEAGUE_NBA,
    client: NBAClient | None = None,
) -> list[dict[str, Any]]:
    """Return league standings (v3).

    Parameters
    ----------
    season: ``"2024-25"`` format.

    Example
    -------
    ::

        rows = league.standings("2024-25")
        for team in rows:
            print(team["TeamName"], team["WINS"], team["LOSSES"])
    """
    http = client or get_client()
    data = http.get(
        "leaguestandingsv3",
        LeagueID=league_id,
        Season=season,
        SeasonType=season_type,
    )
    rs = parse_result_sets(data)
    return rs.get("Standings", [])  # type: ignore[return-value]


def leaders(
    stat: str = "PTS",
    *,
    season: str = "2024-25",
    season_type: str = "Regular Season",
    scope: str = "S",
    per_mode: str = "PerGame",
    league_id: str = LEAGUE_NBA,
    client: NBAClient | None = None,
) -> list[dict[str, Any]]:
    """Return league scoring/stat leaders.

    Parameters
    ----------
    stat:    ``"PTS"``, ``"REB"``, ``"AST"``, ``"STL"``, ``"BLK"``, ``"FG_PCT"``.
    scope:   ``"S"`` = season (default), ``"RS"`` = rookies.

    Example
    -------
    ::

        top_scorers = league.leaders("PTS", season="2024-25")
    """
    http = client or get_client()
    data = http.get(
        "leagueleaders",
        LeagueID=league_id,
        PerMode=per_mode,
        Scope=scope,
        Season=season,
        SeasonType=season_type,
        StatCategory=stat,
    )
    rs = parse_result_sets(data)
    return rs.get("LeagueLeaders", [])  # type: ignore[return-value]


def draft(
    season: str,
    *,
    league_id: str = LEAGUE_NBA,
    client: NBAClient | None = None,
) -> list[dict[str, Any]]:
    """Return draft history for a season.

    Parameters
    ----------
    season: Season end year as 4-digit string e.g. ``"2024"``.
    """
    http = client or get_client()
    data = http.get("drafthistory", LeagueID=league_id, Season=season)
    rs = parse_result_sets(data)
    return rs.get("DraftHistory", [])  # type: ignore[return-value]



"""ESPN advanced endpoint functions — injuries, odds, play-by-play, CDN."""

from __future__ import annotations

from typing import Any

from sportly.client import SportlyClient, get_client
from sportly.espn._domains import ESPNDomain, build_url


def injuries(
    sport: str,
    league: str,
    *,
    client: SportlyClient | None = None,
) -> list[dict[str, Any]]:
    """League-wide injury report."""
    http = client or get_client()
    url = build_url(ESPNDomain.SITE, f"sports/{sport}/{league}/injuries")
    data = http.get(url)
    return data.get("items", data.get("injuries", []))  # type: ignore[return-value]


def transactions(
    sport: str,
    league: str,
    *,
    client: SportlyClient | None = None,
) -> list[dict[str, Any]]:
    """Recent signings, trades, and waivers."""
    http = client or get_client()
    url = build_url(ESPNDomain.SITE, f"sports/{sport}/{league}/transactions")
    data = http.get(url)
    return data.get("items", data.get("transactions", []))  # type: ignore[return-value]


def odds(
    sport: str,
    league: str,
    event_id: str,
    competition_id: str | None = None,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Betting odds for a game from the core API.

    Includes lines from Caesars (38), FanDuel (37), DraftKings (41), BetMGM (58).
    """
    http = client or get_client()
    comp = competition_id or event_id
    url = build_url(
        ESPNDomain.CORE,
        f"sports/{sport}/leagues/{league}/events/{event_id}/competitions/{comp}/odds",
    )
    return http.get(url)  # type: ignore[return-value]


def play_by_play(
    sport: str,
    league: str,
    event_id: str,
    competition_id: str | None = None,
    *,
    limit: int = 300,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Play-by-play data for a game from the core API."""
    http = client or get_client()
    comp = competition_id or event_id
    url = build_url(
        ESPNDomain.CORE,
        f"sports/{sport}/leagues/{league}/events/{event_id}/competitions/{comp}/plays",
    )
    return http.get(url, params={"limit": limit})  # type: ignore[return-value]


def win_probability(
    sport: str,
    league: str,
    event_id: str,
    competition_id: str | None = None,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """In-game win probability data from the core API."""
    http = client or get_client()
    comp = competition_id or event_id
    url = build_url(
        ESPNDomain.CORE,
        f"sports/{sport}/leagues/{league}/events/{event_id}/competitions/{comp}/probabilities",
    )
    return http.get(url)  # type: ignore[return-value]


def cdn_game(
    sport: str,
    game_id: str,
    *,
    client: SportlyClient | None = None,
) -> dict[str, Any]:
    """Full CDN game package (drives, plays, win prob, odds, boxscore).

    Result key ``gamepackageJSON`` contains the richest available game data.
    """
    http = client or get_client()
    url = build_url(ESPNDomain.CDN, f"{sport}/game")
    return http.get(url, params={"xhr": "1", "gameId": game_id})  # type: ignore[return-value]



"""fotmob.endpoints.leagues — league tables, all leagues directory."""
from __future__ import annotations
from typing import Any
from sportly.fotmob.endpoints._client import FotMobClient, get_client


def league(league_id: int | str, *, client: FotMobClient | None = None) -> dict[str, Any]:
    """League data: standings table, recent results, upcoming fixtures.

    Parameters
    ----------
    league_id: Numeric FotMob league ID — see :data:`sportly.fotmob.LEAGUES`.

    Example::

        from sportly.fotmob.endpoints import leagues
        epl = leagues.league(47)
        table = epl["table"][0]["data"]["table"]["all"]
    """
    return (client or get_client()).get("leagues", id=league_id)  # type: ignore[return-value]


def all_leagues(*, client: FotMobClient | None = None) -> dict[str, Any]:
    """Full directory of every competition tracked by FotMob."""
    return (client or get_client()).get("allLeagues")  # type: ignore[return-value]


def tv_listings(*, date: str | None = None, client: FotMobClient | None = None) -> dict[str, Any]:
    """TV broadcast schedule.

    Parameters
    ----------
    date: ``YYYYMMDD`` (optional, defaults to today on remote).
    """
    return (client or get_client()).get("tvlistings", date=date)  # type: ignore[return-value]



"""Tests for sportly.mlb — all mocked, no network."""
from __future__ import annotations
import json
import httpx
import pytest
from sportly.mlb._client import MLBClient

# ── Shared fixtures ───────────────────────────────────────────────────────────

TEAMS_RESPONSE = {
    "teams": [
        {"id": 119, "name": "Los Angeles Dodgers", "abbreviation": "LAD",
         "teamName": "Dodgers", "locationName": "Los Angeles",
         "league": {"name": "National League"}, "division": {"name": "NL West"}}
    ]
}
ROSTER_RESPONSE = {
    "roster": [
        {"person": {"id": 660271, "fullName": "Shohei Ohtani"}, "jerseyNumber": "17",
         "position": {"name": "Infielder", "abbreviation": "1B"}, "status": {"description": "Active"}}
    ]
}
SCHEDULE_RESPONSE = {
    "dates": [
        {
            "date": "2025-04-01",
            "games": [
                {"gamePk": 745444, "gameDate": "2025-04-01T22:10:00Z",
                 "status": {"detailedState": "Final"},
                 "teams": {
                     "home": {"team": {"name": "Los Angeles Dodgers"}, "score": 7},
                     "away": {"team": {"name": "San Diego Padres"}, "score": 2},
                 }}
            ],
        }
    ]
}
BOXSCORE_RESPONSE = {
    "teams": {"home": {"team": {"id": 119}, "teamStats": {}},
              "away": {"team": {"id": 135}, "teamStats": {}}},
    "officials": [], "info": [],
}
PBP_RESPONSE = {
    "allPlays": [
        {"result": {"event": "Strikeout", "description": "K"}, "about": {"inning": 1}},
    ]
}
STANDINGS_RESPONSE = {
    "records": [
        {"standingsType": "regularSeason", "league": {"id": 103},
         "division": {"name": "AL West"},
         "teamRecords": [{"team": {"name": "Houston Astros"}, "wins": 90, "losses": 72}]}
    ]
}
LEADERS_RESPONSE = {
    "leagueLeaders": [
        {"leaderCategory": "homeRuns", "leaders": [
            {"rank": 1, "value": "58", "person": {"fullName": "Aaron Judge"}}
        ]}
    ]
}
PLAYER_RESPONSE = {
    "people": [
        {"id": 660271, "fullName": "Shohei Ohtani", "primaryNumber": "17",
         "primaryPosition": {"name": "Designated Hitter"},
         "currentTeam": {"id": 119}, "active": True}
    ]
}


def _mock(status: int, body: dict):
    """Return an MLBClient backed by a mock transport."""
    def handler(request: httpx.Request) -> httpx.Response:
        return httpx.Response(status, json=body)
    transport = httpx.MockTransport(handler)
    client = MLBClient()
    client._http = httpx.Client(transport=transport)
    return client


# ── Teams ─────────────────────────────────────────────────────────────────────

class TestMLBTeams:
    def test_all_teams(self):
        from sportly.mlb.endpoints import teams
        c = _mock(200, TEAMS_RESPONSE)
        result = teams.all(client=c)
        assert len(result) == 1
        assert result[0]["abbreviation"] == "LAD"

    def test_single_team(self):
        from sportly.mlb.endpoints import teams
        c = _mock(200, {"teams": [TEAMS_RESPONSE["teams"][0]]})
        t = teams.one(119, client=c)
        assert t["id"] == 119

    def test_roster(self):
        from sportly.mlb.endpoints import teams
        c = _mock(200, ROSTER_RESPONSE)
        r = teams.roster(119, client=c)
        assert r[0]["person"]["fullName"] == "Shohei Ohtani"


# ── Schedule ──────────────────────────────────────────────────────────────────

class TestMLBSchedule:
    def test_games_returns_flat_list(self):
        from sportly.mlb.endpoints import schedule
        c = _mock(200, SCHEDULE_RESPONSE)
        result = schedule.games(date="2025-04-01", client=c)
        assert isinstance(result, list)
        assert result[0]["gamePk"] == 745444

    def test_games_empty_dates(self):
        from sportly.mlb.endpoints import schedule
        c = _mock(200, {"dates": []})
        result = schedule.games(date="2025-01-01", client=c)
        assert result == []

    def test_today_delegates(self):
        from sportly.mlb.endpoints import schedule
        c = _mock(200, SCHEDULE_RESPONSE)
        result = schedule.today(client=c)
        assert len(result) == 1


# ── Players ───────────────────────────────────────────────────────────────────

class TestMLBPlayers:
    def test_player_profile(self):
        from sportly.mlb.endpoints import players
        c = _mock(200, PLAYER_RESPONSE)
        p = players.player(660271, client=c)
        assert p["fullName"] == "Shohei Ohtani"

    def test_player_not_found(self):
        from sportly.mlb.endpoints import players
        from sportly.exceptions import NotFoundError
        c = _mock(404, {})
        with pytest.raises(NotFoundError):
            players.player(999999, client=c)

    def test_search(self):
        from sportly.mlb.endpoints import players
        c = _mock(200, PLAYER_RESPONSE)
        result = players.search("Ohtani", client=c)
        assert result[0]["id"] == 660271


# ── Games ─────────────────────────────────────────────────────────────────────

class TestMLBGames:
    def test_boxscore(self):
        from sportly.mlb.endpoints import games
        c = _mock(200, BOXSCORE_RESPONSE)
        box = games.boxscore(745444, client=c)
        assert "teams" in box

    def test_play_by_play(self):
        from sportly.mlb.endpoints import games
        c = _mock(200, PBP_RESPONSE)
        pbp = games.play_by_play(745444, client=c)
        assert pbp["allPlays"][0]["result"]["event"] == "Strikeout"


# ── Stats / League ────────────────────────────────────────────────────────────

class TestMLBStats:
    def test_standings(self):
        from sportly.mlb.endpoints import stats
        c = _mock(200, STANDINGS_RESPONSE)
        records = stats.standings(season=2025, client=c)
        assert records[0]["teamRecords"][0]["wins"] == 90

    def test_leaders(self):
        from sportly.mlb.endpoints import stats
        c = _mock(200, LEADERS_RESPONSE)
        result = stats.leaders("homeRuns", season=2025, client=c)
        assert result[0]["leaderCategory"] == "homeRuns"
        assert result[0]["leaders"][0]["person"]["fullName"] == "Aaron Judge"

    def test_transactions(self):
        from sportly.mlb.endpoints import stats
        payload = {"transactions": [
            {"id": 1, "typeDesc": "Trade", "description": "Player A traded", "date": "2025-03-01"}
        ]}
        c = _mock(200, payload)
        tx = stats.transactions(start_date="2025-03-01", end_date="2025-03-10", client=c)
        assert tx[0]["typeDesc"] == "Trade"

    def test_venues(self):
        from sportly.mlb.endpoints import stats
        payload = {"venues": [{"id": 22, "name": "Dodger Stadium"}]}
        c = _mock(200, payload)
        result = stats.venues(venue_id=22, client=c)
        assert result[0]["name"] == "Dodger Stadium"


# ── Top-level module API ──────────────────────────────────────────────────────

class TestMLBModule:
    def test_module_imports(self):
        import sportly.mlb as mlb
        assert callable(mlb.teams)
        assert callable(mlb.schedule)
        assert callable(mlb.player)
        assert callable(mlb.boxscore)
        assert callable(mlb.leaders)
        assert callable(mlb.standings)
        assert "LAD" in mlb.TEAM_IDS
        assert mlb.TEAM_IDS["LAD"] == 119

    def test_module_teams(self):
        import sportly.mlb as mlb
        c = _mock(200, TEAMS_RESPONSE)
        result = mlb.teams(client=c)
        assert result[0]["abbreviation"] == "LAD"

    def test_module_schedule(self):
        import sportly.mlb as mlb
        c = _mock(200, SCHEDULE_RESPONSE)
        result = mlb.schedule(date="2025-04-01", client=c)
        assert result[0]["gamePk"] == 745444



"""ESPN teams endpoint functions."""

from __future__ import annotations

from typing import Any

from sportly.client import SportlyClient, get_client
from sportly.espn._domains import ESPNDomain, build_url
from sportly.espn._parse import parse_athlete, parse_team
from sportly.models import Athlete, Team


def all(
    sport: str,
    league: str,
    *,
    limit: int = 100,
    client: SportlyClient | None = None,
) -> list[Team]:
    """Return all teams for a sport/league."""
    http = client or get_client()
    url = build_url(ESPNDomain.SITE, f"sports/{sport}/{league}/teams")
    data = http.get(url, params={"limit": limit})
    raw_teams = (
        data.get("sports", [{}])[0]
            .get("leagues", [{}])[0]
            .get("teams", [])
    )
    return [parse_team(t) for t in raw_teams]


def one(
    sport: str,
    league: str,
    team_id: str,
    *,
    client: SportlyClient | None = None,
) -> Team:
    """Return a single team by ESPN team ID."""
    http = client or get_client()
    url = build_url(ESPNDomain.SITE, f"sports/{sport}/{league}/teams/{team_id}")
    data = http.get(url)
    return parse_team(data.get("team", data))


def roster(
    sport: str,
    league: str,
    team_id: str,
    *,
    client: SportlyClient | None = None,
) -> list[Athlete]:
    """Return the full roster for a team."""
    http = client or get_client()
    url = build_url(ESPNDomain.SITE, f"sports/{sport}/{league}/teams/{team_id}/roster")
    data = http.get(url)
    raw: list[dict[str, Any]] = []
    for group in data.get("athletes", []):
        if isinstance(group, dict):
            raw.extend(group.get("items", []))
        elif isinstance(group, list):
            raw.extend(group)
    return [parse_athlete(a) for a in raw]
